use async_trait::async_trait;
use libpt::log::debug;
use octocrab;

use super::{PublishContext, ReleaseContext, ServerApi};
use crate::{
    config::{Api, Config},
    error::*,
};
pub struct Github {
    api: Api,
}

impl Github {
    pub async fn build(api: &Api, _cfg: &Config) -> Result<Self> {
        Ok(Self {
            api: api.to_owned(),
        })
    }
}

#[async_trait]
impl ServerApi for Github {
    async fn push_release(&mut self, rc: ReleaseContext) -> Result<()> {
        debug!("release context: {rc:#?}");
        let t1 = octocrab::instance();
        let t2 = t1.repos(rc.username, rc.repository);
        let t = t2.releases();
        let req = t
            .create(&rc.tag)
            .target_commitish(&rc.commit_sig)
            .name(&rc.tag)
            .body(&rc.text)
            .draft(rc.draft)
            .prerelease(rc.prerelease);
        req.send().await.map_err(ServerApiError::from)?;
        Ok(())
    }
    async fn push_release_artifact(&mut self, _rc: ReleaseContext) -> Result<()> {
        todo!()
    }
    async fn push_pkg(&mut self, _pc: PublishContext) -> Result<()> {
        todo!()
    }
    fn get_inner(&self) -> &Api {
        &self.api
    }
}
